TITLE AboutDialog.asm
;                                      ====================
;======================================= AboutDialog.asm  ==========================================
;==                                    ====================                                       ==
;==                                                                                               ==
;== Project         : ScangII                                                                     ==
;== Filename        : AboutDialog.asm                                                             ==
;== Version         : 1                                                                           ==
;== Date            : Jul 15, 2008                                                                 ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                                      ==
;== Purpose         : Handles the about dialog box and its actions in ScanGII                     ==
;==                                                                                               ==
;===================================================================================================
;==                                                                                               ==
;== Copyright (c) 2008, P. H. Duarte                                                              ==
;== All rights reserved.                                                                          ==
;==                                                                                               ==
;===================================================================================================

; VERSION HISTORY:
; ----------------

; Version           : 1
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : Original Version.
;===================================================================================================

.686
.model flat, stdcall
option casemap:none

;===================================================================================================
;=                                     Exported Symbols and functions                              =
;===================================================================================================

PUBLIC	DLG_ABOUT
PUBLIC	DlgProc

;===================================================================================================
;=                                     Imported Symbols                                            =
;===================================================================================================

EXTERN	szAppName:BYTE

;===================================================================================================
;=                                     Include Files                                               =
;===================================================================================================

include ScanGII.inc ;*
include inc\structures.inc




;===================================================================================================
;=                                  Local Constants                                                =
;===================================================================================================

.const
DLG_ABOUT						equ 2500	; About dialog box ID.
STC_COPYRIGHT					equ 2501	; Static control for copyright message.
STC_VORTEX_SITE					equ 2502	; Button to open Vortex' site
STC_MASM_JUICE					equ 2503	; Static control for code message.
STC_GET_SCAN					equ 2504	; Static control for message.
;===================================================================================================
;=                                  Local Data                                                     =
;===================================================================================================

.data
align 02
szGetScan		TCHAR	"Get scan.exe at:", 0

align 02

szJuiceStatic	TCHAR	"-- This is a 100% MASM juice project. --", 13, 10, \
						"-- Coded to help coders to code. --", 13, 10, \
						"-- Good luck! --", 0 ; About program message.
						
szStaticText	TCHAR 	"-- A gui for Vortex' scan. --", 13, 10, \ ; CopyRight message.
						"-- Copyright 2008  ", 169, 13, 10, " Paulo H. --", 0
align 02
szUrl			TCHAR	"www.vortex.masmcode.com", 0 ; Vortex's site.

align 02
szHttp			TCHAR	"http://", 0

align 02
szAbout			TCHAR	"About ", 0

.data?
bMouseTouch		BOOL		?	; Stores the mouse movement state.
hOldUrlProc		HINSTANCE	?	; Handle to the original DialogBox procedure.
hStcMasm		HANDLE		?	; Static control for code message.
hUrlBrush		HBRUSH		?	; Handle to url static control brush type.
hUrlFontU		HFONT		?	; Handle to underlined font.
hUrlOldFont		HFONT		?	; Handle to original font in url static control.

;===================================================================================================

.code

;===================================================================================================
;
; Function UrlProc(hWin, uMsg, wParam, lParam )
;
; Purpose: Make a static control clickable and open internet browser on a specified link.
;
; Entry:		hWin	= Handle window.
;				uMsg	= Message to process.
;				wParam	= Handle to the device context for the static control window.
;				lParam	= Handle to the static control. 
;
; Description:	The function takes a control and processes messages generated by
;				this control. It changes the color of a clickable text and opens an URL
;				into default web browser.	
;
; Returns:		FALSE if a message was processed otherwise it call CallWindowProc and returns
;				its value
;
; Error Handling:	NA
;
; Important remark: This function was adapted from RadAsm source code with kind permission of
;					KetilO.
;=================================================================================================== 

UrlProc PROC hWin:HWND, uMsg:UINT, wParam: WPARAM, lParam:LPARAM
	LOCAL rect:RECT
	LOCAL szBuffer[sizeof szHttp + sizeof szUrl + 1]:TCHAR
	
	.if dword ptr ds:[uMsg] == WM_SETCURSOR
		invoke LoadCursor, NULL, IDC_HAND
		invoke SetCursor, EAX
	
	.elseif dword ptr ds:[uMsg] == WM_MOUSEMOVE
		invoke GetClientRect, dword ptr ds:[hWin], addr rect
		invoke GetCapture

		.if EAX != dword ptr ds:[hWin]
			mov byte ptr ds:[bMouseTouch], TRUE
			invoke SetCapture, dword ptr ds:[hWin]
			invoke SendMessage, dword ptr ds:[hWin], WM_SETFONT, dword ptr ds:[hUrlFontU], TRUE
		.endif

		mov EDX, dword ptr ds:[lParam]
		movzx EAX, DX
		shr EDX, 16
		
		.if EAX > dword ptr ds:[rect.right] || EDX > dword ptr ds:[rect.bottom]
			mov byte ptr ds:[bMouseTouch], FALSE
			invoke ReleaseCapture
			invoke SendMessage, dword ptr ds:[hWin], WM_SETFONT, dword ptr ds:[hUrlOldFont], TRUE
		.endif
		
		.elseif dword ptr ds:[uMsg] == WM_LBUTTONUP
			mov byte ptr ds:[bMouseTouch],FALSE
			invoke ReleaseCapture
			invoke SendMessage, dword ptr ds:[hWin],WM_SETFONT,hUrlOldFont,TRUE
			invoke RtlZeroMemory, addr szBuffer, sizeof szBuffer ; Clears buffer area.
			invoke lstrcat, addr szBuffer, addr szHttp
			invoke lstrcat, addr szBuffer, addr szUrl
			lea EBX, dword ptr ds:[szBuffer]
			mov dword ptr ds:[SheInfo.lpFile], EBX
			mov dword ptr ds:[SheInfo.nShow], SW_SHOWMAXIMIZED
		invoke ShellExecuteEx, addr SheInfo
	
	.else
		invoke CallWindowProc,
				dword ptr ds:[hOldUrlProc],
				dword ptr ds:[hWin],
				dword ptr ds:[uMsg],
				dword ptr ds:[wParam],
				dword ptr ds:[lParam]
		ret
	.endif
	xor EAX, EAX
	ret

UrlProc endp

;================================================================================
;
; DlgProc (hDlgAbout, uMsg, wParam, lParam)
;
; PURPOSE:	To show a About dialog box with informations about the program.
;
; ENTRY:	hDlgAbout	=	Handle dialog box.
;			uMsg		=	Message to process.
;			wParam		=	Additional parameter.
;			lParam		=	Additional parameter.
;
; EXIT:		TRUE		=	A message was processed.
;			FALSE		=	No message was processed.
;==================================================================================		

DlgProc PROC hDlgAbout:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM
	LOCAL 	hItem:HANDLE
	LOCAL 	hDc:HDC
	LOCAL 	lf:LOGFONT
	LOCAL	szBuffer[sizeof szAbout + sizeof szAppName + 1]:TCHAR

	
	.if uMsg == WM_INITDIALOG
		invoke RtlZeroMemory, addr szBuffer, sizeof szBuffer ; Clears buffer area.
		invoke lstrcat, addr szBuffer, addr szAbout
		invoke lstrcat, addr szBuffer, addr szAppName
		invoke SetWindowText, dword ptr ds:[hDlgAbout], addr szBuffer

; Write on static controls 		

		invoke SendDlgItemMessage, dword ptr ds:[hDlgAbout], STC_MASM_JUICE, WM_SETTEXT, 0, addr szJuiceStatic

		invoke SendDlgItemMessage, dword ptr ds:[hDlgAbout], STC_COPYRIGHT, WM_SETTEXT, 0, addr szStaticText
		
		invoke SendDlgItemMessage, dword ptr ds:[hDlgAbout], STC_GET_SCAN, WM_SETTEXT, 0, addr szGetScan
		
		invoke GetDlgItem, dword ptr ds:[hDlgAbout], STC_VORTEX_SITE
		mov dword ptr ds:[hItem], EAX
		
		invoke SendMessage, dword ptr ds:[hItem], WM_GETFONT, 0, 0
		mov dword ptr ds:[hUrlOldFont], EAX
		
		invoke GetObject, dword ptr ds:[hUrlOldFont], sizeof LOGFONT, addr lf
		mov dword ptr ds:[lf.lfUnderline], TRUE
		
		invoke CreateFontIndirect, addr lf
		mov dword ptr ds:[hUrlFontU], EAX
		
		invoke SendDlgItemMessage, dword ptr ds:[hDlgAbout], STC_VORTEX_SITE, WM_SETTEXT, 0, addr szUrl
		invoke SetWindowLong, dword ptr ds:[hItem], GWL_WNDPROC, addr UrlProc
		mov dword ptr ds:[hOldUrlProc], EAX
		
		invoke GetSysColor, COLOR_3DFACE
		invoke CreateSolidBrush, EAX
		mov dword ptr ds:[hUrlBrush], EAX
				
	.elseif uMsg == WM_COMMAND		
	
	 .elseif uMsg == WM_CTLCOLORSTATIC
		push dword ptr ds:[lParam]	; Handle to static control.
		pop dword ptr ds:[hItem]

		push dword ptr ds:[wParam]	; wParam -> Handle to device context for
									; the static control window.
		pop dword ptr ds:[hDc]

		invoke GetDlgItem, dword ptr ds:[hDlgAbout], STC_VORTEX_SITE
		mov ECX, EAX
		xor EAX, EAX
		
		.if ECX == dword ptr ds:[hItem]

			.if byte ptr ds:[bMouseTouch] != FALSE
				mov EAX, 0FF0000h	; This color is Blue.
			.endif			
						
			invoke SetTextColor, dword ptr ds:[hDc], EAX ; Changes the text color.
			invoke SetBkMode, dword ptr ds:[hDc], TRANSPARENT
			mov eax, dword ptr ds:[hUrlBrush]
		
		.endif
		ret

	.elseif uMsg == WM_CLOSE
		invoke DeleteObject, dword ptr ds:[hUrlFontU]
		invoke DeleteObject, dword ptr ds:[hUrlBrush]
		invoke EndDialog, dword ptr ds:[hDlgAbout], FALSE
			
	.else
		mov EAX, FALSE
		ret
		
	.endif
	mov EAX, TRUE
	ret
	
DlgProc ENDP
end
;===================================================================================================
;=                                             EOF                                                 =
;===================================================================================================