TITLE ScanGII.asm
;                                      ====================
;=======================================   ScanGII.asm    ==========================================
;==                                    ====================                                       ==
;==                                                                                               ==
;== Project         : ScanGui                                                                     ==
;== Filename        : ScanGuII.asm                                                                ==
;== Version         : 2                                                                           ==
;== Date            : Jul 15, 2008                                                                ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                                      ==
;== Purpose         : This is a GUI for Vortex' Scan tool                                         ==
;==                                                                                               ==
;===================================================================================================
;==                                                                                               ==
;== Copyright (c) 2008, P. H. Duarte                                                              ==
;== All rights reserved.                                                                          ==
;==                                                                                               ==
;===================================================================================================

; VERSION HISTORY:
; ----------------

; Version           : 2
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : Changed the main dialog box design.
;                     New About Dialog Box added.
;                     There was a bug when scan.exe was executed. There is no such bug anymore!
;					  Modularized design of program.
;					  Created 03 methods of compilation ( describe below ).
;===================================================================================================

;How to compile:
;
;	For RadAsm users:
;		If you use RadAsm, just open the ScanGII.rap and build it.
;       The executable is ScangII.exe
;
;	For that ones who don't use RadAsm:
;		Use the nmakemakefile with NMAKE like this:
;			NMAKE /f nmakefile
;
;		Or you can use the mingw32-make this way:
;			mingw32-make.exe -fgnumakefile
;
;		Or you can run build.bat:
;			build.bat	
;
;	All of them assumes this folder structure:
;		C:\masm32
;===================================================================================================

; The module AboutDialog.asm has been coded with part of RadAsm code with
; kind permission of KetilO!

;===================================================================================================


.686
.model flat, stdcall
option casemap :none

;===================================================================================================
;=                                      Included Files                                             =
;===================================================================================================

MAINFILE EQU <1>
include	ScanGII.inc
include inc\structures.inc

;===================================================================================================
;=                                   Local Functions                                               =
;===================================================================================================

WinMain                         	PROTO	:HINSTANCE,:HINSTANCE,:LPSTR,:DWORD
WndProc                         	PROTO	:HWND,:UINT,:WPARAM,:LPARAM

;===================================================================================================
;=                                     Imported Symbols and functions                              =
;===================================================================================================

EXTERN	DLG_ABOUT:ABS
EXTERN	DlgProc:PROTO		:HWND, :UINT, :WPARAM, :LPARAM
EXTERN	ChooseFile:PROTO	:HWND
EXTERN	Executa:PROTO		:HWND, :DWORD
EXTERN	StartCombo:PROTO	:HWND
EXTERN	Clean:PROTO			:HWND
EXTERN	MontaLinha:PROTO	:HWND
EXTERN	Verify:PROTO		:HWND
EXTERN	MessagierProc:PROTO	:UINT, :DWORD, :DWORD

;===================================================================================================
;=                                     Exported Symbols and functions                              =
;===================================================================================================

PUBLIC  szAppName
PUBLIC 	szScanPath
PUBLIC	hCombo
PUBLIC	hCheck
PUBLIC  szOperationOpen
PUBLIC	szOutName
PUBLIC	hNewProcess
PUBLIC	hInstApp
PUBLIC	hInstance

;===================================================================================================
;=                                      Local Data                                                 =
;===================================================================================================

.const
;SCanIIMenu.mnu
IDR_MENU						equ 10000
IDM_FILE						equ 10001
IDM_CHOOSE_FILE					equ 10002
IDM_EXIT						equ 10004
IDM_EDIT						equ 10005
IDM_EDIT_SCAN_INI				equ 10006
IDM_RESET						equ 10007
IDM_HELP						equ 10008
IDM_ABOUT						equ 10009
IDM_READ_ME						equ 10010

;Res\ScanGIIAcl.rc
AccelTable						equ 2000
VK_IDM_EXIT						equ 2020
VK_IDM_EDIT_SCAN_INI			equ 2025
VK_IDM_SCAN_PATH				equ 2030
VK_IDM_RESET					equ 2035
VK_IDM_READ_ME					equ 2045
VK_IDM_CHOOSE_FILE				equ 2050

.data?
hCheck			HANDLE 		?	; Handle to check box controls.
hCombo			HANDLE 		?	; Handle to combo box.
hCursor			HANDLE 		?	; Handle to cursor.
hDlg			DWORD		?	; Handle to the main dialog box.
hInstance		HINSTANCE 	?	; Handle to the current instance of the application
lpCmdLine		LPSTR		?	; Pointer to GetCommandLine.
pidl			DWORD		?	; Armazena espao para caminho de Scan.exe.
hNewProcess 	HANDLE		?   ; Handle to the process created with ShellExecuteInfoEx.
hInstApp		HINSTANCE	?	; Handle to the program executed by ShellExecuteInfoEx.

.data
szAppMenuName	TCHAR 	"ScanGuiMenu", 0
szAppClassName	TCHAR 	"DLGCLASS", 0
szAppName		TCHAR	"ScanGui", 0
szAttention		TCHAR 	"Attention!", 0		; Title for attention message boxes.
szBrowseTitle	TCHAR	"Choose scan path:", 0
szError			TCHAR	"Error!", 0			; Title for error message boxes.
szFileReadMe	TCHAR	"Readme.txt", 0
szDlgAbout		TCHAR	"DLG_ABOUT", 0
szDlgMain		TCHAR	"DLG_MAIN", 0
szScanini		TCHAR	"scan.ini", 0
szOperationOpen TCHAR	"open", 0			; Operation to execute.
szOutName		TCHAR	MAXLEN		DUP (0) ; Buffer for imported functions file name.
szCurrentDir	TCHAR	MAXLEN		DUP (0) ; Buffer for current folder name.
szScanPath		TCHAR	MAXLEN		DUP	(0) ; Buffer for Scan.exe folder name. 

;===================================================================================================
;=                                   Local Sructures                                               =
;===================================================================================================

icex	  		INITCOMMONCONTROLSEX <SIZEOF INITCOMMONCONTROLSEX,\
									ICC_USEREX_CLASSES +\
									ICC_UPDOWN_CLASS + \
									ICC_HOTKEY_CLASS + \
									ICC_STANDARD_CLASSES>

browseinfo		BROWSEINFO 		<>

;===================================================================================================

.code
align 04
start:
	invoke GetModuleHandle, NULL
	mov dword ptr ds:[hInstance], EAX
	invoke InitCommonControlsEx, addr icex
	invoke GetCommandLine
	mov dword ptr ds:[lpCmdLine], EAX
	inc EAX
	push EAX
	invoke lstrlen, EAX
	std
	pop EDI
	push EDI
	add EDI, EAX
	xchg EAX, ECX
	mov AL, "\"
	repne scasb
	pop EDI
	add ECX, 3	; ECX -> 03 positions less than we need.
	cld			; For Windows API's work correctly.
	invoke lstrcpyn, addr szCurrentDir, EDI, ECX
	invoke WinMain, dword ptr ds:[hInstance], NULL, dword ptr ds:[lpCmdLine], SW_SHOWDEFAULT
	invoke ExitProcess, 0
	ret

;===================================================================================================
;
; WinMain (hInst, hPrevInst, CmdLine, CmdShow)
;
; PURPOSE:	Funo que inicia o programa.
;
; ENTRY:	hInst		=	Handle to the current instance of the application
;			hPrevInst	=	Handler para a instncia anterior da aplicao.
;			CmdLine		=	Pointer para a string que apresenta a linha de comando da aplicao.
;			CmdShow		=
;
;			 executada quando o usurio clica em Help->About
;			na janela aberta aparece uma breve mensagem sobre o programa, seu
;			propsito e seu autor. Pode ser fechada de maneira usual.
;
; EXIT:		EAX:
;				TRUE	= Uma mensagem foi processada pela janela.
;				FALSE	= Nenhuma mensagem foi processada pela janela.
;===================================================================================================
	
WinMain PROC hInst:HINSTANCE, hPrevInst:HINSTANCE, CmdLine:LPSTR, CmdShow:DWORD
LOCAL wc:WNDCLASSEX
LOCAL msg:MSG
LOCAL hAccel:HACCEL	
	
;	Initialize WNDCLASSEX STRUCTURE			
	
	mov dword ptr ds:[wc.cbSize], sizeof WNDCLASSEX
	mov dword ptr ds:[wc.style], CS_HREDRAW or CS_VREDRAW
	mov dword ptr ds:[wc.lpfnWndProc], offset WndProc
	mov dword ptr ds:[wc.cbClsExtra], 0
	mov dword ptr ds:[wc.cbWndExtra], DLGWINDOWEXTRA
	push dword ptr ds:[hInst]
	pop dword ptr ds:[wc.hInstance]
	mov dword ptr ds:[wc.hbrBackground], COLOR_BTNFACE+1
	mov dword ptr ds:[wc.lpszMenuName], IDR_MENU
	mov dword ptr ds:[wc.lpszClassName], offset szAppClassName
	invoke LoadIcon, NULL, IDI_APPLICATION
	mov dword ptr ds:[wc.hIcon], EAX
	mov dword ptr ds:[wc.hIconSm], EAX
	invoke LoadCursor, NULL, IDC_ARROW
	mov dword ptr ds:[wc.hCursor], EAX
	invoke LoadCursor, NULL, IDC_HAND
	mov dword ptr ds:[hCursor], EAX
	invoke RegisterClassEx, addr wc
	
	.if (EAX == 0) ; Error checking.
		invoke MessagierProc, addr szError, ID_ST_WINDOWVERERROR, MB_OK
		ret
	.endif

	invoke CreateDialogParam, dword ptr ds:[hInstance], addr szDlgMain, NULL, addr WndProc, 0
	
	.if (EAX == NULL)  ; Error checking
		invoke MessagierProc, addr szError, ID_ST_DIALOG_FAILED, MB_ICONWARNING
		ret
	.endif
	
	mov dword ptr ds:[hDlg], EAX
	invoke ShowWindow, dword ptr ds:[hDlg], SW_SHOWNORMAL
	invoke UpdateWindow, dword ptr ds:[hDlg]
	invoke GetDlgItem, dword ptr ds:[hDlg], EDT_1
	invoke SetFocus, EAX

;	Load keyboard accelerators.
 
	invoke LoadAccelerators, dword ptr ds:[hInst], AccelTable
	mov dword ptr ds:[hAccel], EAX

;	Message loop.	
	.while TRUE
		invoke GetMessage, addr msg, NULL, 0, 0
		.break .if ( EAX == 0)
		
		.if EAX == -1
			invoke MessagierProc, addr szError, ID_ST_MESSAGES_ERROR, MB_ICONINFORMATION
			invoke ExitProcess, 0
		.endif
		
		invoke TranslateAccelerator, dword ptr ds:[hDlg], dword ptr ds:[hAccel], addr msg
		
		.if !EAX
			invoke IsDialogMessage, dword ptr ds:[hDlg], addr msg
			
			.if !EAX
				invoke TranslateMessage, addr msg
				invoke DispatchMessage, addr msg
			
			.endif
		
		.endif
	
	.endw
	
;	Frees window space.
	
	invoke UnregisterClass, offset szAppClassName, dword ptr ds:[hInstance]
	.if (!EAX)
		invoke MessagierProc, addr szError, ID_ST_DIALOG_FAILED, MB_OK
	.endif
	mov EAX, dword ptr ds:[msg.wParam]
	ret
WinMain ENDP
	
WndProc PROC hwnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM
LOCAL pt:POINT
	
	.if dword ptr ds:[uMsg]  == WM_INITDIALOG

; --> Initialize BROWSEINFO STRUCTURE. <--
		push dword ptr ds:[hwnd]
		pop dword ptr ds:[browseinfo.hwndOwner]
		mov dword ptr ds:[browseinfo.pidlRoot], NULL
		mov dword ptr ds:[browseinfo.pszDisplayName], offset szScanPath
		mov dword ptr ds:[browseinfo.lpszTitle], offset szBrowseTitle
		push uFlagsBinfo
		pop dword ptr ds:[browseinfo.ulFlags]
	
; 	Get ComboBox handler.
	
		invoke GetDlgItem, dword ptr ds:[hwnd], CBO_SWITCH
		mov dword ptr ds:[hCombo], EAX
		invoke StartCombo, dword ptr ds:[hwnd]
	
	.elseif dword ptr ds:[uMsg] ==  WM_COMMAND
	
; --> Menu commands. <--
	
		LOWORD (wParam)
		.if dword ptr ds:[lParam] == 0
			.if EAX  == IDM_ABOUT
				invoke DialogBoxParam, 
								dword ptr ds:[hInstance], 
								addr szDlgAbout, 
								dword ptr ds:[hwnd], 
								addr DlgProc, 
								0
				
			.elseif EAX == IDM_CHOOSE_FILE || EAX == VK_IDM_CHOOSE_FILE
				invoke ChooseFile, dword ptr ds:[hwnd]	; Choose file to be scanned.
				
				.if EAX	!= 0			; Verify error.
					invoke MessagierProc, addr szAttention, EAX, MB_ICONWARNING ; Shows error message.		
	
; -->Shows selected filename and its path. <--
				
				.else
					invoke SetDlgItemText, dword ptr ds:[hwnd], EDT_1, addr lpstrFile
					invoke SetDlgItemText, dword ptr ds:[hwnd], EDT_2, addr szOutName
				.endif

; --> Open Scan.ini for text edition. <--			
			
			.elseif EAX == IDM_EDIT_SCAN_INI || EAX == VK_IDM_EDIT_SCAN_INI
				mov dword ptr ds:[SheInfo.lpFile], offset szScanini 
				invoke SetCurrentDirectory, addr szCurrentDir
				invoke ShellExecuteEx, addr SheInfo
				
			.elseif EAX == IDM_EXIT	|| EAX == VK_IDM_EXIT	; Quits program.
				invoke DestroyWindow, dword ptr ds:[hwnd] 
				
			.elseif EAX == IDM_READ_ME || EAX == VK_IDM_READ_ME
				mov dword ptr ds:[SheInfo.lpFile], offset szFileReadMe
				invoke SetCurrentDirectory, addr szCurrentDir
				invoke ShellExecuteEx, addr SheInfo
				
			.elseif EAX == IDM_RESET	|| EAX == VK_IDM_RESET	; Reset values.
				invoke Clean, dword ptr ds:[hwnd]	

				
;			.elseif EAX == IDM_SCAN_PATH	|| EAX == VK_IDM_SCAN_PATH ; Procura o executvel scan.exe.
;				invoke CoInitializeEx, NULL, COINIT_APARTMENTTHREADED
;				
;				.if  EAX != S_OK		; Verify erro de CoInitializeEx: S_OK == sem erros!!!.
;					xor EAX, EAX
;					ret
;				.endif
;				
;				invoke SHBrowseForFolder, offset browseinfo
;				mov dword ptr ds:[pidl], EAX
;				
;	Copia caminho de scan.exe para a varivel szScanPath
;				invoke RtlZeroMemory, addr szScanPath, MAXLEN
;				invoke SHGetPathFromIDList, dword ptr ds:[pidl], addr szScanPath
;				invoke CoTaskMemFree, dword ptr ds:[pidl]
;				invoke CoUninitialize
				
			.endif
; --> Menu messages end. <-- 	
		.else
			push EAX
			HIWORD (wParam)
			xchg EAX, EDX
			pop EAX
			
			.if EDX == BN_CLICKED
				
				.if EAX == BTN_CHOOSE 	; Choose source file.
					invoke SendMessage, dword ptr ds:[hwnd], WM_COMMAND, IDM_CHOOSE_FILE, NULL
					
				.elseif EAX == BTN_EXIT	; Quit this boring program.
					invoke SendMessage, dword ptr ds:[hwnd], WM_COMMAND, IDM_EXIT, 0
					
; --> Generates the .imp extension. <-- 
				.elseif EAX == BTN_GEN	
					invoke Verify, dword ptr ds:[hwnd]
					
					.if EAX
						invoke MessagierProc, addr szError, EAX, MB_ICONEXCLAMATION
						
					.else
						invoke MontaLinha, dword ptr ds:[hwnd] 
						invoke Executa, dword ptr ds:[hwnd], EDI ; EDI -> Command line to scan.exe.
						.if EAX > 0	; Verifies errors.
							invoke MessagierProc, addr szAttention, EAX, MB_ICONEXCLAMATION
						.endif
					.endif
				.endif
			.endif
		.endif
	
	.elseif dword ptr ds:[uMsg] == WM_CONTEXTMENU
		mov		eax, dword ptr ds:[wParam]
		.if eax==dword ptr ds:[hwnd]
			mov	eax, dword ptr ds:[lParam]
			push eax
			and	eax, 0FFFFh
			mov	dword ptr ds:[pt.x], eax
			pop eax
			shr	eax, 16
			mov	dword ptr ds:[pt.y], eax
			invoke GetMenu, dword ptr ds:[hwnd]
			invoke GetSubMenu, EAX, 1
	    	invoke TrackPopupMenu, 
	    					EAX, 
	    					TPM_LEFTALIGN or TPM_RIGHTBUTTON, 
	    					dword ptr ds:[pt.x], 
	    					dword ptr ds:[pt.y], 
	    					0, 
	    					dword ptr ds:[hwnd], 
	    					0
		.endif
	
	.elseif dword ptr ds:[uMsg] ==  WM_DESTROY
		invoke PostQuitMessage, 0	; Quit this boring program!
	
	.elseif dword ptr ds:[uMsg] == WM_CLOSE	; Quit this boring program.
		invoke DestroyWindow, hwnd 
	
	.else
	invoke DefWindowProc,
				dword ptr ds:[hwnd],
				dword ptr ds:[uMsg],
				dword ptr ds:[wParam],
				dword ptr ds:[lParam]
	ret
	
	.endif
	
	xor EAX, EAX
	ret
	
WndProc ENDP
	

end start
;===================================================================================================
;=                                         EOF                                                     =
;===================================================================================================